<?php
namespace nur\sery\db;

/**
 * Class CapacitorChannel: un canal d'une instance de {@link ICapacitor}
 */
class CapacitorChannel {
  const NAME = null;

  const EACH_COMMIT_THRESHOLD = 100;

  static function verifix_name(?string $name): string {
    if ($name === null) $name = "default";
    return strtolower($name);
  }

  function __construct(?string $name=null, ?int $eachCommitThreshold=null) {
    $this->name = self::verifix_name($name ?? static::NAME);
    $this->eachCommitThreshold = $eachCommitThreshold ?? static::EACH_COMMIT_THRESHOLD;
    $this->created = false;
  }

  /** @var string */
  protected $name;

  function getName(): string {
    return $this->name;
  }

  /**
   * @var ?int nombre maximum de modifications dans une transaction avant un
   * commit automatique dans {@link Capacitor::each()}. Utiliser null pour
   * désactiver la fonctionnalité.
   */
  protected $eachCommitThreshold;

  function getEachCommitThreshold(): ?int {
    return $this->eachCommitThreshold;
  }

  function getTableName(): string {
    return $this->name."_channel";
  }

  protected $created;

  function isCreated(): bool {
    return $this->created;
  }

  function setCreated(bool $created=true): void {
    $this->created = $created;
  }

  /**
   * retourner un ensemble de définitions pour des colonnes supplémentaires à
   * insérer lors du chargement d'une valeur
   *
   * la colonne "_id" de définition "integer primary key autoincrement" est la
   * clé primaire par défaut. elle peut être redéfinie, et dans ce cas la valeur
   * à utiliser doit être retournée par {@link getKeyValues()}
   */
  function getKeyDefinitions(): ?array {
    return null;
  }

  /**
   * calculer les valeurs des colonnes supplémentaires à insérer pour le
   * chargement de $item
   *
   * Si "_id" est retourné, la ligne existante est mise à jour le cas échéant.
   */
  function getKeyValues($item): ?array {
    return null;
  }

  /**
   * méthode appelée lors du chargement d'un élément avec
   * {@link Capacitor::charge()}
   *
   * @param mixed $item l'élément à charger
   * @param array $values les valeurs calculées par {@link getKeyValues()}
   * @param ?array $row la ligne à mettre à jour. vaut null s'il faut insérer
   * une nouvelle ligne
   * @return ?array le cas échéant, un tableau non null à marger dans $values et
   * utiliser pour provisionner la ligne nouvelle créée, ou mettre à jour la
   * ligne existante
   *
   * Si $item est modifié dans cette méthode, il est possible de le retourner
   * avec la clé "_item" pour mettre à jour la ligne correspondante
   */
  function onCharge($item, array $values, ?array $row): ?array {
    return null;
  }

  /**
   * méthode appelée lors du parcours des éléments avec
   * {@link Capacitor::each()}
   *
   * @param mixed $item l'élément courant
   * @param ?array $row la ligne à mettre à jour
   * @return ?array le cas échéant, un tableau non null utilisé pour mettre à
   * jour la ligne courante
   *
   * Si $item est modifié dans cette méthode, il est possible de le retourner
   * avec la clé "_item" pour mettre à jour la ligne correspondante
   */
  function onEach($item, array $row): ?array {
    return null;
  }
}
